/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("net.jazz.ajax.ui._Application");

// TODO: Do not require Workbench here (see Jazz Enhancement #32401)
dojo.require("net.jazz.ajax.ui.Workbench");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.declare("net.jazz.ajax.ui._Application", null, {
	
	// summary:
	//		Abstract superclass that provides the basic capabilities of a
	//		Jazz Ajax Framework-based application
	// description:
	//		Subclasses must minimally do the following:
	//		
	//		1) Define an 'init' method to perform whatever initialization logic
	//		   they require.
	//		2) Call the 'start' method of this class via the extender's
	//		   'superclass' property;
	//		   e.g. my.extender.superclass.start.call(this);
	
	start: function() {
		// summary:
		//		informs the workbench to begin processing events; extenders must
		//		call this method exactly once per lifecycle, usually at the end
		//		of the 'init' method
		
		// create URI monitor and process first URI
		net.jazz.ajax.ui.PlatformUI.getWorkbench()._start();
	},
	
	setName: function(name) {
		// summary:
		//		sets the name for the application; the workbench will prepend
		//		the application name before the contents of the argument to
		//		Workbench#setTitle
		name = dojo.trim(name);
		if(!name){
			throw new Error("_Application#setName: Missing required paameter 'name'");
		}
		net.jazz.ajax.ui.PlatformUI.getWorkbench()._setDefaultTitle(name);
	},
	
	//
	// Internal methods; do not call directly
	//
	
	constructor: function(args) {
		if (!dojo.isFunction(this.init)) {
			alert(NO_INIT_METHOD_MSG);
			throw new Error(NO_INIT_METHOD_MSG);
		}
		this.init();
	}
	
});

var NO_INIT_METHOD_MSG = "Cannot start application (no 'init' method found).";

})();