/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("net.jazz.ajax.ui.internal.PageRegistry");

dojo.require("net.jazz.ajax.core.runtime.RegistryFactory");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.PlatformUI");

/* 
Eclipse IPerspectiveRegistry supports:

 clonePerspective(String id, String label, IPerspectiveDescriptor desc)      Clones an existing perspective.
 IPerspectiveDescriptor 	findPerspectiveWithId(String perspectiveId)   Finds and returns the registered perspective with the given perspective id.
 IPerspectiveDescriptor 	findPerspectiveWithLabel(String label) Finds and returns the registered perspective with the given label.
 String 	getDefaultPerspective()Returns the id of the default perspective for the workbench.
 IPerspectiveDescriptor[] 	getPerspectives()  Returns a list of the perspectives known to the workbench.
 void 	revertPerspective(IPerspectiveDescriptor perspToRevert) Reverts a perspective back to its original definition as specified in the plug-in manifest.
 void 	setDefaultPerspective(String id) 

*/

var checkNotNull 	= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 	= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var RegistryFactory = net.jazz.ajax.core.runtime.RegistryFactory;

var removeIf 		= net.jazz.ajax.toolbox.lang.removeIf;
var findIf			= net.jazz.ajax.toolbox.lang.findIf;

var sortFunction	= null;

function getPageExtensionPoint() {
	var extensionPoint = RegistryFactory.getRegistry().getExtensionPoint("net.jazz.ajax.pages");
	
	if(!extensionPoint) {
		throw new Error("Page extension point not found");
	}
	
	return extensionPoint;
}

function getPage(extension, configElement) {
	checkNotNull(extension, "extension");
	checkNotNull(configElement, "configElement");
	
	checkNotNull(extension.getNamespace(), "extension namespace");
	checkNotNull(configElement.getAttribute("id"), "page id");
	checkNotNull(configElement.getAttribute("name"), "page name");
	
	//dojo.lang.assert(configElement.getName() == "page", "invalid configuration element: " + configElement.getName());

	var page = {
		id: configElement.getAttribute("id"),
		name: configElement.getAttribute("name"),
		href: configElement.getAttribute("href"),
		widget: configElement.getAttribute("widget"),
		defaultAction: configElement.getAttribute("defaultAction"),
		removalPropertyId: configElement.getAttribute("removalPropertyId"),
		component: extension.getNamespace(),
		getUri: function() { return "#action=jazz.viewPage&id=" + this.id },
		actions: []
	};
	
	dojo.forEach(configElement.getChildren(), function(childCfgElement) {
		if (configElement.getName() !== "action") {
			return;
		}
		checkNotNull(childCfgElement.id);
		getPage.actions.push({});
	});

	return page;
}

function getPages() {
	var workbenchId  = net.jazz.ajax.ui.PlatformUI._getApplicationId();
	var pageArray = [];

	dojo.forEach( getPageExtensionPoint().getExtensions(), function(extension) {
		dojo.forEach(extension.getConfigurationElements(), function(configElement) {
			var scopeFound = false;
			var inScope = false;
			dojo.forEach(configElement.getChildren(), function(childCfgElement) {
				if (childCfgElement.getName() !== "applicationScope") {
					return;
				}
				scopeFound = true;
				checkNotNull(childCfgElement.getAttribute("id"));
				if (childCfgElement.getAttribute("id") == workbenchId) {
					inScope = true;
				}
			});
			if (!scopeFound && !inScope) {
				// No scope was specified so assume that the page is available for all applications
				inScope = true;
			}
			if (inScope) {
				var pushPage = true;
				var page = getPage(extension, configElement);
				if (page.removalPropertyId != null) {
					var removalProperty = dojo.getObject(page.removalPropertyId);
					if (removalProperty && removalProperty == true) {
						pushPage = false;
					}
				}
				if (pushPage) {
					pageArray.push(page);
				}
			}
		});
	});
	
	if(sortFunction){
		pageArray.sort(sortFunction);
	}
	
	return pageArray;
}

net.jazz.ajax.ui.internal.PageRegistry.getPages = getPages;

function findPageWithId(id) {
	checkNotNull(id, "id");
	
	var matchedPage = null;
	
	dojo.forEach(getPages(), function(page) {
		if(page.id == id)
			matchedPage = page;
	});
	
	return matchedPage;
}

net.jazz.ajax.ui.internal.PageRegistry.findPageWithId = findPageWithId;

function setSortFunction(/*Function*/sortFunctionArg){
	if(!dojo.isFunction(sortFunctionArg)){
		throw new Error("PageRegistry#setSortFunction: argument is not a function");
	}
	sortFunction = sortFunctionArg;
}

net.jazz.ajax.ui.internal.PageRegistry.setSortFunction = setSortFunction;

})();