/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyElement");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.internal.HoverPopup");

(function() {

var HoverPopup = net.jazz.ajax.ui.internal.HoverPopup;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyElement", null, {
	
	_createInputNode: function () {
		var node;
		if (this.elementInfo.propertyType == "BOOLEAN") {
			node = document.createElement("select");
			var option = document.createElement("option");
			option.text = "true";
			option.value = "true";
			node.options[node.options.length] = option;
			option = document.createElement("option");
			option.text = "false";
			option.value = "false";
			node.options[node.options.length] = option;
			node._propertyElement = this;
			node.className = "editValueNode";
			node.value = this._currentValue;
		}
		else {
			node = document.createElement("input");
			node._propertyElement = this;
			if (this.elementInfo.accessPolicy == "SENSITIVE") {
				node.type = "password";
			}
			else {
				node.type = "text";
			}
			node.className = "editValueNode";
			node.value = this._currentValue;
		}
		return node;
	},
	
	_show: function () {
		this.domNode.style.display = "";
		this._showing = true;
		
		if (!this._initialized) {
			var beforeNode = this.displayIn.firstChild;
			while (beforeNode != null && this._sortFunction(this.domNode, beforeNode) > 0) {
				beforeNode = beforeNode.nextSibling;
			}
			if (beforeNode == null) {
				this.displayIn.appendChild(this.domNode);
			}
			else {
				this.displayIn.insertBefore(this.domNode, beforeNode);
			}
			this._initialized = true;
			if (this._descriptionMode == "tooltip") {
				// The tooltip sometimes has problems attaching to the object if without a small delay
				setTimeout(dojo.hitch(this, function() {
					 this._addToolTip();
				}), 50);
			}
		}
	},
	
	_sortFunction: function (a, b) {
		if (a._customSortVal != null || b._customSortVal != null) {
			if (a._customSortVal != null && b._customSortVal != null) {
				if (a._customSortVal < b._customSortVal) {
					return -1;
				}
				else if (a._customSortVal > b._customSortVal) {
					return 1;
				}
				else {
					return 0;
				}
			}
			else if (a._customSortVal == null) {
				return 1;
			}
			else {
				return -1;
			}
		}
		else {
			if (a._defaultSortVal < b._defaultSortVal) {
				return -1;
			}
			else if (a._defaultSortVal > b._defaultSortVal) {
				return 1;
			}
			else {
				return 0;
			}
		}
	},
	
	_hide: function () {
		this.domNode.style.display = "none";
		this._showing = false;
	},
	
	_addToolTip: function () {
		if (this._description != null && this._description != "") {
			this._propertyName.appendChild(document.createElement("span"));
			var tooltip = new HoverPopup({
				connectId: [ this._propertyName.id ],
				label: this._description,
				showDelay: 300
			}, this._propertyName.lastChild);
		}
	},
	
	_markChanged: function () {
		this._setCSSClass("changePending", true);
		this._changed = true;
	},
	
	_markUnchanged: function () {
		this._setCSSClass("changePending", false);
		this._changed = false;
		var propChange = false;
	},
	
	_checkValidity: function () {
		this._validationPending = true;
	},
	
	_markValid: function () {
		while (this._invalidValueMsg.childNodes.length > 0) {
			this._invalidValueMsg.removeChild(this._invalidValueMsg.firstChild);
		}
		this._errIcon.style.display = "";
		this._errIcon.style.visibility = "hidden";
		this._warnIcon.style.display = "none";
		this._invalid = false;
	},
	
	_markInvalid: function (msg) {
		this._invalid = true;
		while (this._invalidValueMsg.childNodes.length > 0) {
			this._invalidValueMsg.removeChild(this._invalidValueMsg.firstChild);
		}
		
		this._invalidValueMsg.appendChild(document.createTextNode(msg.message));
		if (msg.code === "ERROR"){
			this._invalidValueMsg.className = "invalidMsg";
			this._errIcon.style.visibility = "visible";
			this._warnIcon.style.display = "none";
		} else {
			this._invalidValueMsg.className = "invalidWarnMsg";
			this._errIcon.style.display = "none";
			this._warnIcon.style.display = "inline";
		}
	},
	
	_setCSSClass: function (className, activate) {
		this._cssHash[className] = activate;
		var cssSet = false;
		for (var i = 0; i < this._cssPriority.length; i++) {
			if (this._cssHash[this._cssPriority[i]]) {
				this.domNode.className = this._cssPriority[i];
				cssSet = true;
				break;
			}
		}
		if (!cssSet) {
			for (cssClass in this._cssHash) {
				if (this._cssHash[cssClass]) {
					this.domNode.className = cssClass;
					cssSet = true;
					break;
				}
			}
		}
		if (!cssSet) {
			this.domNode.className = "";
		}
	},
	
	_validateChanges: function () {
		if (this._invalid) {
			return false;
		}
		else {
			return true;
		}
	},
	
	_getChange: function () {
		return {
			name: this._name,
			value: this._editValueNode.value,
			serviceImplementationName: this.implementationClass
		};
	},
	
	_handleSuccessfulChange: function () {
		this._markUnchanged();
		this.domNode.className = "";
		this._setCSSClass("successfulChange", true);
		if (this._restartRequired) {
			setTimeout(dojo.hitch(this, function () {
				this._setCSSClass("restartRequired", true);
				this._setCSSClass("successfulChange", false);
				// Display "Server Restart Required" message if it isn't already showing
				if (dojo.byId(this._name + "_restartMsg") == null) {
					var div = document.createElement("div");
					div.id = this._name + "_restartMsg";
					div.className = "serverRestart";
					div.appendChild(document.createTextNode("(Server Restart Required)"));
					this._viewValueNode.parentNode.appendChild(div);
				}
			}), 1000);
		}
		else {
			setTimeout(dojo.hitch(this, function () {
				this._setCSSClass("successfulChange", false);
			}), 1000);
			this._currentValue = this._editValueNode.value;
		}
		this._savedValue = this._editValueNode.value;
	}, 
	
	revertDefault: function () {
		// set edit node to default
		this._editValueNode.value = this.elementInfo.defaultValue != null ? this.elementInfo.defaultValue : "";
		this._handleValueChange();
	}, 
	
	restoreSaved:  function () {
		// set edit node to saved value
		this._editValueNode.value = this._savedValue;
		this._handleValueChange();
	}, 
	
	// moving this out of onchange inline fucntion so revert/restore can trigger it too
	_handleValueChange: function() {
			if (this._editValueNode.value != this._savedValue) {
				this._markChanged();
			}
			else {
				this._markUnchanged();
			}
			if (this._editValueNode.value == this._defaultValue) {
				dojo.toggleClass(this._propertyDefault, "fadedText", true);
			}
			else {
				dojo.toggleClass(this._propertyDefault, "fadedText", false);
			}
			while (this._viewValueNode.childNodes.length > 0) {
				this._viewValueNode.removeChild(this._viewValueNode.childNodes[0]);
			}
			if (this.elementInfo.accessPolicy == "SENSITIVE") {
				var displayVal = this._editValueNode.value;
				this._viewValueNode.appendChild(document.createTextNode(displayVal.replace(/./g,"*")));
			}
			else {
				this._viewValueNode.appendChild(document.createTextNode(this._editValueNode.value));
			}
	}, 
	
	hasOverride: function () {
		return this._editValueNode.value != this.elementInfo.defaultValue && this._editValueNode.value != "";
	}

});

})();